@@ -53,7 +53,8 @@ SUPPORTED_VENDORS = (
     'dell',
     'foundry',
     'juniper',
-    'netscreen'
+    'netscreen',
+    'paloalto',
 )
 VALID_VENDORS = SUPPORTED_VENDORS # For backwards compatibility
 
@@ -87,6 +88,7 @@ SUPPORTED_PLATFORMS = {
     'foundry': ['ROUTER', 'SWITCH'],
     'juniper': ['FIREWALL', 'ROUTER', 'SWITCH'],  # Any devices running Junos
     'netscreen': ['FIREWALL'],                    # Pre-Juniper NetScreens
+    'paloalto': ['FIREWALL'],
 }
 
 # The tuple of support device types
@@ -105,6 +107,7 @@ DEFAULT_TYPES = {
     'foundry': 'SWITCH',
     'juniper': 'ROUTER',
     'netscreen': 'FIREWALL',
+    'paloalto': 'FIREWALL',
 }
 
 # When a vendor is not explicitly defined within `DEFAULT_TYPES`, fallback to
@@ -136,9 +139,8 @@ SSH_PTY_DISABLED = {
 # would like to disable asynchronous (NON-interactive) SSH sessions, such as
 # when using twister or Commando to remotely control a device.
 SSH_ASYNC_DISABLED = {
-    'arista': ['SWITCH'],  # Known not to work w/ SSH ... yet
-    'brocade': ['SWITCH'], # Namely the Brocade VDX =(
     'dell': ['SWITCH'],    # Dell SSH is just straight up broken
+    'foundry': ['SWITCH'], # Old Foundry switches only do SSHv1
 }
 
 # Vendors that basically just emulate Cisco's IOS and can be treated
@@ -152,6 +154,28 @@ IOSLIKE_VENDORS = (
     'foundry',
 )
 
+# The file path where .gorc is expected to be found.
+GORC_FILE = '~/.gorc'
+
+# The only root commands that are allowed to be executed when defined within
+# ``~.gorc``. They will be filtered # out by `~trigger.gorc.filter_commands()`.
+GORC_ALLOWED_COMMANDS = (
+    'cli',
+    'exit',
+    'get',
+    'monitor',
+    'ping',
+    'quit',
+    'set',
+    'show',
+    'start',
+    'term',
+    'terminal',
+    'traceroute',
+    'who',
+    'whoami'
+)
+
 #===============================
 # NetDevices
 #===============================
@@ -160,14 +184,28 @@ IOSLIKE_VENDORS = (
 # 'from trigger.acl.autoacl import autoacl' without modifying sys.path.
 AUTOACL_FILE = os.environ.get('AUTOACL_FILE', os.path.join(PREFIX, 'autoacl.py'))
 
-# One of 'xml', 'json', 'sqlite'. This MUST match the actual format of
-# NETDEVICES_FILE or it won't work for obvious reasons.
-NETDEVICES_FORMAT = os.environ.get('NETDEVICES_FORMAT', 'xml')
+# A tuple of data loader classes, specified as strings. Optionally, a tuple can
+# be used instead of a string. The first item in the tuple should be the
+# Loader's module, subsequent items are passed to the Loader during
+# initialization.
+NETDEVICES_LOADERS = (
+    'trigger.netdevices.loaders.filesystem.XMLLoader',
+    'trigger.netdevices.loaders.filesystem.JSONLoader',
+    'trigger.netdevices.loaders.filesystem.SQLiteLoader',
+    'trigger.netdevices.loaders.filesystem.CSVLoader',
+    'trigger.netdevices.loaders.filesystem.RancidLoader',
+    # Example of a database loader where the db information is sent along as an
+    # argument. The args can be anything you want.
+    #['trigger.netdevices.loaders.mysql.Loader', {'dbuser': 'root', 'dbpass': 'abc123', 'dbhost': 'localhost', 'dbport': 3306}, 'bacon'],
+)
 
-# Path to netdevices device metadata source file, which is used to populate
-# trigger.netdevices.NetDevices. This may be JSON, XML, or a SQLite3 database.
-# You must set NETDEVICES_FORMAT to match the type of data.
-NETDEVICES_FILE = os.environ.get('NETDEVICES_FILE', os.path.join(PREFIX, 'netdevices.xml'))
+# A path or URL to netdevices device metadata source data, which is used to
+# populate trigger.netdevices.NetDevices. For more information on this, see
+# NETDEVICES_LOADERS.
+NETDEVICES_SOURCE = os.environ.get('NETDEVICES_SOURCE', os.path.join(PREFIX, 'netdevices.xml'))
+
+# Assign NETDEVICES_SOURCE to NETDEVICES_FILE for backwards compatibility
+NETDEVICES_FILE = NETDEVICES_SOURCE
 
 # Whether to treat the RANCID root as a normal instance, or as the root to
 # multiple instances. This is only checked when using RANCID as a data source.
@@ -190,6 +228,47 @@ JUNIPER_FULL_COMMIT_FIELDS = {
     'make': 'EX4200',
 }
 
+#===============================
+# Prompt Patterns
+#===============================
+# Specially-defined, per-vendor prompt patterns. If a vendor isn't defined her,
+# try to use IOSLIKE_PROMPT_PAT or fallback to DEFAULT_PROMPT_PAT.
+PROMPT_PATTERNS = {
+    'aruba': r'\(\S+\)(?: \(\S+\))?\s?#',
+    'citrix': r'\sDone\n$',
+    'juniper': r'\S+\@\S+(?:\>|#)\s$',
+    'paloalto': r'\r\n\S+(?:\>|#)\s?',
+    'netscreen': r'(\w+?:|)[\w().-]*\(?([\w.-])?\)?\s*->\s*$',
+}
+
+# When a pattern is not explicitly defined for a vendor, this is what we'll try
+# next (since most vendors are in fact IOS-like)
+IOSLIKE_PROMPT_PAT = r'\S+(\(config(-[a-z:1-9]+)?\))?#'
+
+# Generic prompt to match most vendors. It assumes that you'll be greeted with
+# a "#" prompt.
+DEFAULT_PROMPT_PAT = r'\S+#'
+
+#===============================
+# Bounce Windows/Change Mgmt
+#===============================
+
+# Path of the explicit module file for bounce.py containing custom bounce
+# window mappings.
+BOUNCE_FILE = os.environ.get('BOUNCE_FILE', os.path.join(PREFIX, 'bounce.py'))
+
+# Default bounce timezone. All BounceWindow objects are configured using
+# US/Eastern for now.
+BOUNCE_DEFAULT_TZ = 'US/Eastern'
+
+# The default fallback window color for bounce windows. Must be one of
+# ('green', 'yellow', or 'red').
+#
+#     green: Low risk
+#    yellow: Medium risk
+#       red: High risk
+BOUNCE_DEFAULT_COLOR = 'red'
+
 #===============================
 # Redis Settings
 #===============================
