@@ -53,7 +53,8 @@ SUPPORTED_VENDORS = (
     'dell',
     'foundry',
     'juniper',
-    'netscreen'
+    'netscreen',
+    'paloalto',
 )
 VALID_VENDORS = SUPPORTED_VENDORS # For backwards compatibility
 
@@ -89,6 +90,7 @@ SUPPORTED_PLATFORMS = {
     'foundry': ['ROUTER', 'SWITCH'],
     'juniper': ['FIREWALL', 'ROUTER', 'SWITCH'],  # Any devices running Junos
     'netscreen': ['FIREWALL'],                    # Pre-Juniper NetScreens
+    'paloalto': ['FIREWALL'],
 }
 
 # The tuple of support device types
@@ -108,6 +110,7 @@ DEFAULT_TYPES = {
     'foundry': 'SWITCH',
     'juniper': 'ROUTER',
     'netscreen': 'FIREWALL',
+    'paloalto': 'FIREWALL',
 }
 
 # When a vendor is not explicitly defined within `DEFAULT_TYPES`, fallback to
@@ -139,9 +142,8 @@ SSH_PTY_DISABLED = {
 # would like to disable asynchronous (NON-interactive) SSH sessions, such as
 # when using twister or Commando to remotely control a device.
 SSH_ASYNC_DISABLED = {
-    'arista': ['SWITCH'],  # Known not to work w/ SSH ... yet
-    'brocade': ['SWITCH'], # Namely the Brocade VDX =(
     'dell': ['SWITCH'],    # Dell SSH is just straight up broken
+    'foundry': ['SWITCH'], # Old Foundry switches only do SSHv1
 }
 
 # Vendors that basically just emulate Cisco's IOS and can be treated
@@ -156,6 +158,28 @@ IOSLIKE_VENDORS = (
     'foundry',
 )
 
+# The file path where .gorc is expected to be found.
+GORC_FILE = '~/.gorc'
+
+# The only root commands that are allowed to be executed when defined within
+# ``~.gorc``. They will be filtered # out by `~trigger.gorc.filter_commands()`.
+GORC_ALLOWED_COMMANDS = (
+    'cli',
+    'exit',
+    'get',
+    'monitor',
+    'ping',
+    'quit',
+    'set',
+    'show',
+    'start',
+    'term',
+    'terminal',
+    'traceroute',
+    'who',
+    'whoami'
+)
+
 #===============================
 # NetDevices
 #===============================
@@ -205,6 +229,27 @@ JUNIPER_FULL_COMMIT_FIELDS = {
     #'make': 'EX4200',
 }
 
+#===============================
+# Prompt Patterns
+#===============================
+# Specially-defined, per-vendor prompt patterns. If a vendor isn't defined her,
+# try to use IOSLIKE_PROMPT_PAT or fallback to DEFAULT_PROMPT_PAT.
+PROMPT_PATTERNS = {
+    'aruba': r'\(\S+\)(?: \(\S+\))?\s?#',
+    'citrix': r'\sDone\n$',
+    'juniper': r'\S+\@\S+(?:\>|#)\s$',
+    'paloalto': r'\r\n\S+(?:\>|#)\s?',
+    'netscreen': r'(\w+?:|)[\w().-]*\(?([\w.-])?\)?\s*->\s*$',
+}
+
+# When a pattern is not explicitly defined for a vendor, this is what we'll try
+# next (since most vendors are in fact IOS-like)
+IOSLIKE_PROMPT_PAT = r'\S+(\(config(-[a-z:1-9]+)?\))?#'
+
+# Generic prompt to match most vendors. It assumes that you'll be greeted with
+# a "#" prompt.
+DEFAULT_PROMPT_PAT = r'\S+#'
+
 #===============================
 # Bounce Windows/Change Mgmt
 #===============================
