@@ -39,11 +39,6 @@ from trigger.utils import network, cli
 
 # Constants
 CONTINUE_PROMPTS = ['continue?', 'proceed?', '(y/n):', '[y/n]:']
-DEFAULT_PROMPT_PAT = r'\S+#' # Will match most hardware
-ARUBA_PROMPT_PAT = r'\(\S+\)(?: \(\S+\))?\s?#'
-IOSLIKE_PROMPT_PAT = r'\S+(\(config(-[a-z:1-9]+)?\))?#'
-SCREENOS_PROMPT_PAT = '(\w+?:|)[\w().-]*\(?([\w.-])?\)?\s*->\s*$'
-NETSCALER_PROMPT_PAT = '\sDone\n$' # ' Done \n' only
 
 
 # Functions
@@ -56,20 +51,33 @@ def has_junoscript_error(tag):
         return True
     return False
 
+def has_juniper_error(s):
+    """Test whether a string seems to contain an Juniper error."""
+    tests = (
+        'unknown command.' in s,
+        'syntax error, ' in s,
+        'invalid value.' in s,
+        'missing argument.' in s,
+    )
+    return any(tests)
+
 def has_ioslike_error(s):
     """Test whether a string seems to contain an IOS-like error."""
     tests = (
         s.startswith('%'),                 # Cisco, Arista
-        '\n%' in s,                        # Aruba, Foundry
+        '\n%' in s,                        # A10, Aruba, Foundry
         'syntax error: ' in s,             # Brocade VDX
         s.startswith('Invalid input -> '), # Brocade MLX
     )
-
     return any(tests)
 
 def has_netscaler_error(s):
     """Test whether a string seems to contain a NetScaler error."""
-    return s.startswith('ERROR:')
+    tests = (
+        s.startswith('ERROR: '),
+        '\nERROR: ' in s,
+    )
+    return any(tests)
 
 def is_awaiting_confirmation(prompt):
     """
@@ -78,7 +86,6 @@ def is_awaiting_confirmation(prompt):
     :param prompt:
         The prompt string to check
     """
-    log.msg('Got confirmation prompt: %r' % prompt)
     prompt = prompt.lower()
     matchlist = CONTINUE_PROMPTS
     return any(prompt.endswith(match) for match in matchlist)
@@ -134,7 +141,7 @@ def pty_connect(device, action, creds=None, display_banner=None,
 
     # SSH?
     if device.can_ssh_pty():
-        log.msg('SSH connection test PASSED')
+        log.msg('[%s] SSH connection test PASSED' % device)
         if hasattr(sys, 'ps1') or not sys.stderr.isatty() \
          or not sys.stdin.isatty() or not sys.stdout.isatty():
             # Shell not in interactive mode.
@@ -151,7 +158,8 @@ def pty_connect(device, action, creds=None, display_banner=None,
 
     # or Telnet?
     else:
-        log.msg('SSH connection test FAILED, falling back to telnet')
+        log.msg('[%s] SSH connection test FAILED, falling back to telnet' %
+                device)
         factory = TriggerTelnetClientFactory(d, action, creds,
                                              init_commands=init_commands)
         log.msg('Trying telnet to %s' % device, debug=True)
@@ -266,7 +274,7 @@ def _choose_execute(device, force_cli=False):
         _execute = execute_netscreen
     elif device.vendor == 'juniper':
         if force_cli:
-            _execute = execute_exec_ssh
+            _execute = execute_generic_ssh
         else:
             _execute = execute_junoscript
     else:
@@ -364,7 +372,7 @@ def execute_generic_ssh(device, commands, creds=None, incremental=None,
     if channel_class is None:
         channel_class = TriggerSSHGenericChannel
     if prompt_pattern is None:
-        prompt_pattern = DEFAULT_PROMPT_PAT
+        prompt_pattern = device.vendor.prompt_pattern
     if connection_class is None:
         connection_class = TriggerSSHConnection
 
@@ -410,7 +418,6 @@ def execute_junoscript(device, commands, creds=None, incremental=None,
     Please see `~trigger.twister.execute` for a full description of the
     arguments and how this works.
     """
-
     assert device.vendor == 'juniper'
 
     channel_class = TriggerSSHJunoscriptChannel
@@ -474,41 +481,45 @@ def execute_ioslike_telnet(device, commands, creds=None, incremental=None,
     reactor.connectTCP(device.nodeName, 23, factory)
     return d
 
+def execute_async_pty_ssh(device, commands, creds=None, incremental=None,
+                          with_errors=False, timeout=settings.DEFAULT_TIMEOUT,
+                          command_interval=0, prompt_pattern=None):
+    """
+    Execute via SSH for a device that requires shell + pty-req.
+
+    Please see `~trigger.twister.execute` for a full description of the
+    arguments and how this works.
+    """
+    channel_class = TriggerSSHAsyncPtyChannel
+    method = 'Async PTY'
+    if prompt_pattern is None:
+        prompt_pattern = device.vendor.prompt_pattern
+
+    return execute_generic_ssh(device, commands, creds, incremental,
+                               with_errors, timeout, command_interval,
+                               channel_class, prompt_pattern, method)
+
 def execute_ioslike_ssh(device, commands, creds=None, incremental=None,
                         with_errors=False, timeout=settings.DEFAULT_TIMEOUT,
                         command_interval=0):
     """
     Execute via SSH for IOS-like devices with some exceptions.
 
-    Currently confirmed for A10, Brocade MLX, and Cisco only. For all other
-    IOS-like vendors will use telnet for now. :(
-
     Please see `~trigger.twister.execute` for a full description of the
     arguments and how this works.
     """
     assert device.is_ioslike()
 
-    channel_class = TriggerSSHGenericChannel
-    prompt_pattern = IOSLIKE_PROMPT_PAT
-    method = 'IOS-like'
-
     # Test if device requires shell + pty-req
     if device.requires_async_pty:
-        channel_class = TriggerSSHAsyncPtyChannel
-        method = 'Async PTY'
-
-    # Aruba requires its own prompt pattern
-    if device.vendor == 'aruba':
-        prompt_pattern = ARUBA_PROMPT_PAT
-
-    # Hackery to determine the right "IOS-like" SSH function
-    if device.vendor == 'arista':
-        return execute_exec_ssh(device, commands, creds, incremental,
-                                with_errors, timeout, command_interval)
+        return execute_async_pty_ssh(device, commands, creds, incremental,
+                                     with_errors, timeout, command_interval)
+    # Or fallback to generic
     else:
+        method = 'IOS-like'
         return execute_generic_ssh(device, commands, creds, incremental,
                                    with_errors, timeout, command_interval,
-                                   channel_class, prompt_pattern, method)
+                                   method=method)
 
 def execute_netscreen(device, commands, creds=None, incremental=None,
                       with_errors=False, timeout=settings.DEFAULT_TIMEOUT,
@@ -528,11 +539,10 @@ def execute_netscreen(device, commands, creds=None, incremental=None,
         creds = tacacsrc.get_device_password(device.nodeName)
 
     channel_class = TriggerSSHGenericChannel
-    prompt_pattern = SCREENOS_PROMPT_PAT
     method = 'NetScreen'
     return execute_generic_ssh(device, commands, creds, incremental,
                                with_errors, timeout, command_interval,
-                               channel_class, prompt_pattern, method)
+                               channel_class, method=method)
 
 def execute_netscaler(device, commands, creds=None, incremental=None,
                       with_errors=False, timeout=settings.DEFAULT_TIMEOUT,
@@ -546,11 +556,10 @@ def execute_netscaler(device, commands, creds=None, incremental=None,
     assert device.is_netscaler()
 
     channel_class = TriggerSSHNetscalerChannel
-    prompt_pattern = NETSCALER_PROMPT_PAT
     method = 'NetScaler'
     return execute_generic_ssh(device, commands, creds, incremental,
                                with_errors, timeout, command_interval,
-                               channel_class, prompt_pattern, method)
+                               channel_class, method=method)
 
 
 # Classes
@@ -631,7 +640,7 @@ class TriggerSSHChannelFactory(TriggerClientFactory):
         if connection_class is None:
             connection_class = TriggerSSHConnection
         if prompt_pattern is None:
-            prompt_pattern = DEFAULT_PROMPT_PAT
+            prompt_pattern = settings.DEFAULT_PROMPT_PAT
 
         self.protocol = TriggerSSHTransport
         self.display_banner = None
@@ -809,7 +818,7 @@ class TriggerSSHUserAuth(userauth.SSHUserAuthClient):
                               if meth not in self.authenticatedWith],
                              key=orderByPreference)
 
-        log.msg('can continue with: %s' % canContinue)
+        log.msg('Can continue with: %s' % canContinue)
         log.msg('Already tried: %s' % self.authenticatedWith, debug=True)
         return self._cbUserauthFailure(None, iter(canContinue))
 
@@ -892,11 +901,11 @@ class TriggerSSHMultiplexConnection(TriggerSSHConnection):
             return None
 
         def command_completed(result, chan):
-            log.msg('command completed: %r' % chan.command)
+            log.msg('Command completed: %r' % chan.command)
             return result
 
         def command_failed(failure, chan):
-            log.msg('command failed: %r' % chan.command)
+            log.msg('Command failed: %r' % chan.command)
             return failure
 
         def log_status(result):
@@ -1008,7 +1017,6 @@ class TriggerSSHChannelBase(channel.SSHChannel, TimeoutMixin, object):
                 # etc.
         """
         self.factory = self.conn.transport.factory
-        log.msg('COMMANDS: %r' % self.factory.commands)
         self.commanditer = self.factory.commanditer
         self.results = self.factory.results
         self.with_errors = self.factory.with_errors
@@ -1017,10 +1025,12 @@ class TriggerSSHChannelBase(channel.SSHChannel, TimeoutMixin, object):
         self.prompt = self.factory.prompt
         self.setTimeout(self.factory.timeout)
         self.device = self.factory.device
+        log.msg('[%s] COMMANDS: %r' % (self.device, self.factory.commands))
         self.data = ''
         self.initialized = self.factory.initialized
         self.startup_commands = self.device.startup_commands
-        log.msg('My startup commands: %r' % self.startup_commands, debug=True)
+        log.msg('[%s] My startup commands: %r' % (self.device,
+                                                  self.startup_commands))
 
     def channelOpen(self, data):
         """Do this when the channel opens."""
@@ -1039,16 +1049,16 @@ class TriggerSSHChannelBase(channel.SSHChannel, TimeoutMixin, object):
 
         If the shell never establishes, this won't be called.
         """
-        log.msg('Got channel request response!')
+        log.msg('[%s] Got channel request response!' % self.device)
 
     def _ebShellOpen(self, reason):
-        log.msg('Channel request failed: %s' % reason)
+        log.msg('[%s] Channel request failed: %s' % (self.device, reason))
 
     def dataReceived(self, bytes):
         """Do this when we receive data."""
         # Append to the data buffer
         self.data += bytes
-        log.msg('BYTES: %r' % bytes)
+        log.msg('[%s] BYTES: %r' % (self.device, bytes))
         #log.msg('BYTES: (left: %r, max: %r, bytes: %r, data: %r)' %
         #        (self.remoteWindowLeft, self.localMaxPacket, len(bytes),
         #         len(self.data)))
@@ -1058,7 +1068,7 @@ class TriggerSSHChannelBase(channel.SSHChannel, TimeoutMixin, object):
         if not m:
             #log.msg('STATE: prompt match failure', debug=True)
             return None
-        log.msg('STATE: prompt %r' % m.group(), debug=True)
+        log.msg('[%s] STATE: prompt %r' % (self.device, m.group()))
 
         # Strip the prompt from the match result
         result = self.data[:m.start()]
@@ -1068,10 +1078,10 @@ class TriggerSSHChannelBase(channel.SSHChannel, TimeoutMixin, object):
         if self.initialized:
             self.results.append(result)
 
-        # By default we're checking for IOS-like errors because most vendors
-        # fall under this category.
-        if has_ioslike_error(result) and not self.with_errors:
-            log.msg('ERROR: %r' % result, debug=True)
+        # By default we're checking for IOS-like or Juniper errors because most
+        # vendors # fall under this category.
+        if (has_ioslike_error(result) or has_juniper_error(result)) and not self.with_errors:
+            log.msg('[%s] Command failed: %r' % (self.device, result))
             self.factory.err = exceptions.CommandFailure(result)
             self.loseConnection()
             return None
@@ -1079,8 +1089,8 @@ class TriggerSSHChannelBase(channel.SSHChannel, TimeoutMixin, object):
         # Honor the command_interval and then send the next command
         else:
             if self.command_interval:
-                log.msg('Waiting %s seconds before sending next command' %
-                        self.command_interval)
+                log.msg('[%s] Waiting %s seconds before sending next command' %
+                        (self.device, self.command_interval))
             reactor.callLater(self.command_interval, self._send_next)
 
     def _send_next(self):
@@ -1090,14 +1100,17 @@ class TriggerSSHChannelBase(channel.SSHChannel, TimeoutMixin, object):
         self.resetTimeout()
 
         if not self.initialized:
-            log.msg('Not initialized, sending startup commands', debug=True)
+            log.msg('[%s] Not initialized; sending startup commands' %
+                    self.device)
             if self.startup_commands:
                 next_init = self.startup_commands.pop(0)
-                log.msg('Sending initialize command: %r' % next_init)
+                log.msg('[%s] Sending initialize command: %r' % (self.device,
+                                                                 next_init))
                 self.write(next_init)
                 return None
             else:
-                log.msg('Successfully initialized for command execution')
+                log.msg('[%s] Successfully initialized for command execution' %
+                        self.device)
                 self.initialized = True
 
         if self.incremental:
@@ -1106,7 +1119,8 @@ class TriggerSSHChannelBase(channel.SSHChannel, TimeoutMixin, object):
         try:
             next_command = self.commanditer.next()
         except StopIteration:
-            log.msg('CHANNEL: out of commands, closing...', debug=True)
+            log.msg('[%s] CHANNEL: out of commands, closing connection...' %
+                    self.device)
             self.loseConnection()
             return None
 
@@ -1114,7 +1128,8 @@ class TriggerSSHChannelBase(channel.SSHChannel, TimeoutMixin, object):
             self.results.append(None)
             self._send_next()
         else:
-            log.msg('sending SSH command %r' % next_command, debug=True)
+            log.msg('[%s] Sending SSH command %r' % (self.device,
+                                                     next_command))
             self.write(next_command + '\n')
 
     def loseConnection(self):
@@ -1122,20 +1137,20 @@ class TriggerSSHChannelBase(channel.SSHChannel, TimeoutMixin, object):
         Terminate the connection. Link this to the transport method of the same
         name.
         """
-        log.msg('Forcefully closing transport connection')
+        log.msg('[%s] Forcefully closing transport connection' % self.device)
         self.conn.transport.loseConnection()
 
     def timeoutConnection(self):
         """
         Do this when the connection times out.
         """
-        log.msg('Timed out while sending commands to %s' % self.device)
+        log.msg('[%s] Timed out while sending commands' % self.device)
         self.factory.err = exceptions.CommandTimeout('Timed out while sending commands')
         self.loseConnection()
 
     def request_exit_status(self, data):
         status = struct.unpack('>L', data)[0]
-        log.msg('Exit status: %s' % status)
+        log.msg('[%s] Exit status: %s' % (self.device, status))
 
 class TriggerSSHGenericChannel(TriggerSSHChannelBase):
     """
@@ -1186,7 +1201,7 @@ class TriggerSSHCommandChannel(TriggerSSHChannelBase):
     def channelOpen(self, data):
         """Do this when the channel opens."""
         self._setup_channelOpen()
-        log.msg('Channel was opened')
+        log.msg('[%s] Channel was opened' % self.device)
         d = self.conn.sendRequest(self, 'exec', common.NS(self.command),
                                   wantReply=True)
         d.addCallback(self._gotResponse)
@@ -1196,26 +1211,28 @@ class TriggerSSHCommandChannel(TriggerSSHChannelBase):
         """
         If the shell never establishes, this won't be called.
         """
-        log.msg('CHANNEL %s: Exec finished.' % self.id)
+        log.msg('[%s] CHANNEL %s: Exec finished.' % (self.device, self.id))
         self.conn.sendEOF(self)
 
     def _ebShellOpen(self, reason):
-        log.msg('CHANNEL %s: Channel request failed: %s' % (reason, self.id))
+        log.msg('[%s] CHANNEL %s: Channel request failed: %s' % (self.device,
+                                                                 reason,
+                                                                 self.id))
 
     def dataReceived(self, bytes):
         self.data += bytes
         #log.msg('BYTES INFO: (left: %r, max: %r, bytes: %r, data: %r)' %
         #        (self.remoteWindowLeft, self.localMaxPacket, len(bytes), len(self.data)))
-        log.msg('BYTES RECV: %r' % bytes)
+        log.msg('[%s] BYTES RECV: %r' % (self.device, bytes))
 
     def eofReceived(self):
-        log.msg('CHANNEL %s: EOF received.' % self.id)
+        log.msg('[%s] CHANNEL %s: EOF received.' % (self.device, self.id))
         result = self.data
 
         # By default we're checking for IOS-like errors because most vendors
         # fall under this category.
         if has_ioslike_error(result) and not self.with_errors:
-            log.msg('ERROR: %r' % result, debug=True)
+            log.msg('[%s] Command failed: %r' % (self.device, result))
             self.factory.err = exceptions.CommandFailure(result)
 
         # Honor the command_interval and then send the next command
@@ -1226,30 +1243,31 @@ class TriggerSSHCommandChannel(TriggerSSHChannelBase):
 
     def send_next_command(self):
         """Send the next command in the stack stored on the connection"""
-        log.msg('CHANNEL %s sending next command!' % self.id)
+        log.msg('[%s] CHANNEL %s: sending next command!' % (self.device, self.id))
         self.conn.send_command()
 
     def closeReceived(self):
-        log.msg('CHANNEL %s: Close received.' % self.id)
+        log.msg('[%s] CHANNEL %s: Close received.' % (self.device, self.id))
         self.loseConnection()
 
     def loseConnection(self):
         """Default loseConnection"""
-        log.msg("LOSING CHANNEL CONNECTION")
+        log.msg("[%s] LOSING CHANNEL CONNECTION" % self.device)
         channel.SSHChannel.loseConnection(self)
 
     def closed(self):
-        log.msg('channel %s closed' % self.id)
-        log.msg('CONN CHANNELS: %s' % len(self.conn.channels))
+        log.msg('[%s] Channel %s closed' % (self.device, self.id))
+        log.msg('[%s] CONN CHANNELS: %s' % (self.device,
+                                            len(self.conn.channels)))
 
         # If we're out of channels, shut it down!
         if len(self.conn.transport.factory.results) == len(self.conn.commands):
-            log.msg('RESULTS MATCHES COMMANDS SENT.')
+            log.msg('[%s] RESULTS MATCHES COMMANDS SENT.' % self.device)
             self.conn.transport.loseConnection()
 
     def request_exit_status(self, data):
         exitStatus = int(struct.unpack('>L', data)[0])
-        log.msg('Exit status: %s' % exitStatus)
+        log.msg('[%s] Exit status: %s' % (self.device, exitStatus))
 
 class TriggerSSHJunoscriptChannel(TriggerSSHChannelBase):
     """
@@ -1274,7 +1292,7 @@ class TriggerSSHJunoscriptChannel(TriggerSSHChannelBase):
 
     def dataReceived(self, data):
         """Do this when we receive data."""
-        #log.msg('BYTES: %r' % data, debug=True)
+        log.msg('[%s] BYTES: %r' % (self.device, data))
         self.xmltb.feed(data)
 
     def _send_next(self):
@@ -1286,10 +1304,12 @@ class TriggerSSHJunoscriptChannel(TriggerSSHChannelBase):
 
         try:
             next_command = self.commanditer.next()
-            log.msg('COMMAND: next command=%s' % next_command, debug=True)
+            log.msg('[%s] COMMAND: next command %s' % (self.device,
+                                                       next_command))
 
         except StopIteration:
-            log.msg('CHANNEL: out of commands, closing...', debug=True)
+            log.msg('[%s] CHANNEL: out of commands, closing connection...' %
+                    self.device)
             self.loseConnection()
             return None
 
@@ -1308,6 +1328,7 @@ class TriggerSSHJunoscriptChannel(TriggerSSHChannelBase):
         self.results.append(tag)
 
         if has_junoscript_error(tag) and not self.with_errors:
+            log.msg('[%s] Command failed: %r' % (self.device, tag))
             self.factory.err = exceptions.JunoscriptCommandFailure(tag)
             self.loseConnection()
             return None
@@ -1316,8 +1337,8 @@ class TriggerSSHJunoscriptChannel(TriggerSSHChannelBase):
         # stack
         else:
             if self.command_interval:
-                log.msg('Waiting %s seconds before sending next command' %
-                        self.command_interval)
+                log.msg('[%s] Waiting %s seconds before sending next command' %
+                        (self.device, self.command_interval))
             reactor.callLater(self.command_interval, self._send_next)
 
 class TriggerSSHNetscalerChannel(TriggerSSHChannelBase):
@@ -1331,7 +1352,7 @@ class TriggerSSHNetscalerChannel(TriggerSSHChannelBase):
     def dataReceived(self, bytes):
         """Do this when we receive data."""
         self.data += bytes
-        log.msg('BYTES: %r' % bytes, debug=True)
+        log.msg('[%s] BYTES: %r' % (self.device, bytes))
         #log.msg('BYTES: (left: %r, max: %r, bytes: %r, data: %r)' %
         #        (self.remoteWindowLeft, self.localMaxPacket, len(bytes), len(self.data)))
 
@@ -1340,7 +1361,8 @@ class TriggerSSHNetscalerChannel(TriggerSSHChannelBase):
         if has_netscaler_error(self.data):
             err = self.data
             if not self.with_errors:
-                self.factory.err = exceptions.NetscalerCommandFailure(err)
+                log.msg('[%s] Command failed: %r' % (self.device, err))
+                self.factory.err = exceptions.CommandFailure(err)
                 self.loseConnection()
                 return None
             else:
@@ -1351,7 +1373,7 @@ class TriggerSSHNetscalerChannel(TriggerSSHChannelBase):
         if not m:
             #log.msg('STATE: prompt match failure', debug=True)
             return None
-        log.msg('STATE: prompt %r' % m.group(), debug=True)
+        log.msg('[%s] STATE: prompt %r' % (self.device, m.group()))
 
         result = self.data[:m.start()] # Strip ' Done\n' from results.
 
@@ -1359,8 +1381,8 @@ class TriggerSSHNetscalerChannel(TriggerSSHChannelBase):
             self.results.append(result)
 
         if self.command_interval:
-            log.msg('Waiting %s seconds before sending next command' %
-                    self.command_interval)
+            log.msg('[%s] Waiting %s seconds before sending next command' %
+                    (self.device, self.command_interval))
         reactor.callLater(self.command_interval, self._send_next)
 
 #==================
@@ -1424,19 +1446,22 @@ class TriggerTelnet(telnet.Telnet, telnet.ProtocolTransportMixin, TimeoutMixin):
         Arista Networks hardware is the only vendor that needs this method
         right now.
         """
-        log.msg('TriggerTelnet.enableRemote option: %r' % option, debug=True)
+        #log.msg('[%s] enableRemote option: %r' % (self.host, option))
+        log.msg('enableRemote option: %r' % option)
         return True
 
     def login_state_machine(self, bytes):
         """Track user login state."""
         self.host = self.transport.connector.host
-        log.msg('CONNECTOR HOST: ', self.transport.connector.host)
+        log.msg('[%s] CONNECTOR HOST: %s' % (self.host,
+                                             self.transport.connector.host))
         self.data += bytes
-        log.msg('STATE:  got data %r' % self.data, debug=True)
+        log.msg('[%s] STATE:  got data %r' % (self.host, self.data))
         for (text, next_state) in self.waiting_for:
-            log.msg('STATE:  possible matches %r' % text, debug=True)
+            log.msg('[%s] STATE:  possible matches %r' % (self.host, text))
             if self.data.endswith(text):
-                log.msg('Entering state %r' % next_state.__name__, debug=True)
+                log.msg('[%s] Entering state %r' % (self.host,
+                                                    next_state.__name__))
                 self.resetTimeout()
                 next_state()
                 self.data = ''
@@ -1470,7 +1495,7 @@ class TriggerTelnet(telnet.Telnet, telnet.ProtocolTransportMixin, TimeoutMixin):
         """
         self.setTimeout(None)
         data = self.data.lstrip('\n')
-        log.msg('state_logged_in, DATA: %r' % data, debug=True)
+        log.msg('[%s] state_logged_in, DATA: %r' % (self.host, data))
         del self.waiting_for, self.data
 
         # Run init_commands
@@ -1492,7 +1517,7 @@ class TriggerTelnet(telnet.Telnet, telnet.ProtocolTransportMixin, TimeoutMixin):
         TACACS by default. Use 'aaa authentication login privilege-mode'.
         Also, why no space after the Password: prompt here?
         """
-        log.msg("ENABLE: Sending command: enable\n", debug=True)
+        log.msg("[%s] ENABLE: Sending command: enable" % self.host)
         self.write('enable\n')
         self.waiting_for = [
             ('Password: ', self.state_enable_pw), # Foundry
@@ -1512,7 +1537,7 @@ class TriggerTelnet(telnet.Telnet, telnet.ProtocolTransportMixin, TimeoutMixin):
         if pw is None:
             pw = ''
 
-        log.msg('Sending password %s' % pw, debug=True)
+        #log.msg('Sending password %s' % pw)
         self.write(pw + '\n')
         self.waiting_for = [('>', self.state_enable),
                             ('#', self.state_logged_in),
@@ -1526,7 +1551,7 @@ class TriggerTelnet(telnet.Telnet, telnet.ProtocolTransportMixin, TimeoutMixin):
         else:
             from trigger.netdevices import NetDevices
             pw = NetDevices().find(self.host).enablePW
-        log.msg('Sending password %s' % pw, debug=True)
+        #log.msg('Sending password %s' % pw)
         self.write(pw + '\n')
         self.waiting_for = [('#', self.state_logged_in),
                             ('\n% ', self.state_percent_error),
@@ -1542,13 +1567,13 @@ class TriggerTelnet(telnet.Telnet, telnet.ProtocolTransportMixin, TimeoutMixin):
     def state_raise_error(self):
         """Do this when we get a login failure."""
         self.waiting_for = []
-        log.msg('Failed logging into %s' % self.host)
+        log.msg('Failed logging into %s' % self.transport.connector.host)
         self.factory.err = exceptions.LoginFailure('%r' % self.data.rstrip())
         self.loseConnection()
 
     def timeoutConnection(self):
         """Do this when we timeout logging in."""
-        log.msg('Timed out while logging into %s' % self.host)
+        log.msg('[%s] Timed out while logging in' % self.transport.connector.host)
         self.factory.err = exceptions.LoginTimeout('Timed out while logging in')
         self.loseConnection()
 
@@ -1568,9 +1593,10 @@ class IoslikeSendExpect(protocol.Protocol, TimeoutMixin):
         self.with_errors = with_errors
         self.timeout = timeout
         self.command_interval = command_interval
-        self.prompt =  re.compile(IOSLIKE_PROMPT_PAT)
+        self.prompt =  re.compile(settings.IOSLIKE_PROMPT_PAT)
         self.startup_commands = self.device.startup_commands
-        log.msg('My initialize commands: %r' % self.startup_commands, debug=True)
+        log.msg('[%s] My initialize commands: %r' % (self.device,
+                                                     self.startup_commands))
         self.initialized = False
 
     def connectionMade(self):
@@ -1578,14 +1604,14 @@ class IoslikeSendExpect(protocol.Protocol, TimeoutMixin):
         self.setTimeout(self.timeout)
         self.results = self.factory.results = []
         self.data = ''
-        log.msg('connectionMade, data: %r' % self.data, debug=True)
+        log.msg('[%s] connectionMade, data: %r' % (self.device, self.data))
 
         # Don't call _send_next, since we expect to see a prompt, which
         # will kick off initialization.
 
     def dataReceived(self, bytes):
         """Do this when we get data."""
-        log.msg('BYTES: %r' % bytes, debug=True)
+        log.msg('[%s] BYTES: %r' % (self.device, bytes))
         self.data += bytes
 
         # See if the prompt matches, and if it doesn't, see if it is waiting
@@ -1595,6 +1621,8 @@ class IoslikeSendExpect(protocol.Protocol, TimeoutMixin):
         if not m:
             # If the prompt confirms set the index to the matched bytes,
             if is_awaiting_confirmation(self.data):
+                log.msg('[%s] Got confirmation prompt: %r' % (self.device,
+                                                              self.data))
                 prompt_idx = self.data.find(bytes)
             else:
                 return None
@@ -1607,21 +1635,21 @@ class IoslikeSendExpect(protocol.Protocol, TimeoutMixin):
         # since the telnet session is in WONT ECHO.  This is confirmed with
         # a packet trace, and running self.transport.dont(ECHO) from
         # connectionMade() returns an AlreadyDisabled error.  What's up?
-        log.msg('IoslikeSendExpect.dataReceived result BEFORE: %r' % result, debug=True)
+        log.msg('[%s] result BEFORE: %r' % (self.device, result))
         result = result[result.find('\n')+1:]
-        log.msg('IoslikeSendExpect.dataReceived result AFTER: %r' % result, debug=True)
+        log.msg('[%s] result AFTER: %r' % (self.device, result))
 
         if self.initialized:
             self.results.append(result)
 
         if has_ioslike_error(result) and not self.with_errors:
-            log.msg('ERROR: %r' % result, debug=True)
+            log.msg('[%s] Command failed: %r' % (self.device, result))
             self.factory.err = exceptions.IoslikeCommandFailure(result)
             self.loseConnection()
         else:
             if self.command_interval:
-                log.msg('Waiting %s seconds before sending next command' %
-                        self.command_interval)
+                log.msg('[%s] Waiting %s seconds before sending next command' %
+                        (self.device, self.command_interval))
             reactor.callLater(self.command_interval, self._send_next)
 
     def _send_next(self):
@@ -1630,14 +1658,17 @@ class IoslikeSendExpect(protocol.Protocol, TimeoutMixin):
         self.resetTimeout()
 
         if not self.initialized:
-            log.msg('Not initialized, sending startup commands', debug=True)
+            log.msg('[%s] Not initialized, sending startup commands' %
+                    self.device)
             if self.startup_commands:
                 next_init = self.startup_commands.pop(0)
-                log.msg('Sending initialize command: %r' % next_init)
+                log.msg('[%s] Sending initialize command: %r' % (self.device,
+                                                                 next_init))
                 self.write(next_init)
                 return None
             else:
-                log.msg('Successfully initialized for command execution')
+                log.msg('[%s] Successfully initialized for command execution' %
+                        self.device)
                 self.initialized = True
 
         if self.incremental:
@@ -1646,7 +1677,8 @@ class IoslikeSendExpect(protocol.Protocol, TimeoutMixin):
         try:
             next_command = self.commanditer.next()
         except StopIteration:
-            log.msg('No more commands to send, disconnecting...', debug=True)
+            log.msg('[%s] No more commands to send, disconnecting...' %
+                    self.device)
             self.loseConnection()
             return None
 
@@ -1654,11 +1686,11 @@ class IoslikeSendExpect(protocol.Protocol, TimeoutMixin):
             self.results.append(None)
             self._send_next()
         else:
-            log.msg('Sending command %r to %s' % (next_command, self.device), debug=True)
+            log.msg('[%s] Sending command %r' % (self.device, next_command))
             self.write(next_command + '\n')
 
     def timeoutConnection(self):
         """Do this when we timeout."""
-        log.msg('Timed out while sending commands to %s' % self.device)
+        log.msg('[%s] Timed out while sending commands' % self.device)
         self.factory.err = exceptions.CommandTimeout('Timed out while sending commands')
         self.loseConnection()
